

class MungSubstitutor():

    substitution_table_simple = {
        'a': '@',
        'b': '8',
        'c': '(',
        'd': '6',
        'e': '3',
        'f': '#',
        'g': '9',
        'h': '#',
        'i': '1',
        'j': '!',
        'k': '<',
        'l': '1',
        'o': '0',
        'q': '9',
        's': '$',
        't': '+',
        'v': '>',
        'w': 'uu',
        'x': '%',
        '/': '-',
        '-': '/',
        '0': 'O',
        '1': 'I',
        '3': 'E',
        'A': '@',
        'B': '8',
        'C': '(',
        'E': '3',
        'I': '1',
        'O': '0',
        'S': '2'
    }

    def substitute(self, pwd):
        return self.munger(pwd, self.substitution_table_simple.iteritems())

    def munger(self, pwd, table):
        """
        Function for munging a
        password string
        """
        munged_pwd = pwd
        for letter, replacement in table:
            munged_pwd = munged_pwd.replace(letter, replacement)

        return munged_pwd



from patalib import Antonym, Synonym, Syzygy, Anomaly, Clinamen


class PasswordGenerator:

    bio_data = {}
    key = ""
    password_mappings = {}
    encryption_dict = {}
    substitutors_dict = {}
    words = []

    def __init__(
            self,
            key,
            words,
            bio_data,
            encryption_dict,
            substitutors_dict):
        """
        Store list of biographical data
        """
        self.key = key
        self.words = words
        self.bio_data = bio_data
        self.encryption_dict = encryption_dict
        self.substitutors_dict = substitutors_dict

    def process_individual(self):
        """
        Process an individuals data
        to generate a list of potential
        passwords
        """
        individual = {}
        count = 0
        for k, v in self.bio_data.iteritems():
            individual[self.key] = k
            if not isinstance(v, list):
                individual[k] = self.gen_pata_data(str(v))
            else:
                list_vals_to_process = []
                for listval in v:
                    list_vals_to_process.append(
                        self.gen_pata_data(str(listval)))
                individual[k] = list_vals_to_process

        return individual

    def gen_pata_data(self, bio_val):
        """
        Generate Pata Data
        """
        pata_data = []
        pata_data.append({'original': bio_val})
        pata_data.append(self.synonyms(bio_val))
        pata_data.append(self.antonym(bio_val))
        pata_data.append(self.syzygy(bio_val))
        pata_data.append(self.anomaly(bio_val))
        pata_data.append(self.clinamen(bio_val))

        return pata_data

    def gen_enc_list(self, clear_text):
        """
        Generate a list of encrypted
        items
        """
        encrypted = {}
        temp_enc_list = []

        for e in self.encryption_dict:
            for p in clear_text:
                temp_enc_list.append(self.encryption_dict[e]().hash(p))
            encrypted[e] = temp_enc_list
            temp_enc_list = []

        return encrypted

    def synonyms(self, bio_val):
        """
        Generate synonyms of input data
        """
        synonyms = Synonym().generate_synonym(bio_val)
        synonyms = list(set(synonyms['results']))
        new_synonyms = []
        clear_text = []
        encrypted = {}

        for i in synonyms:
            new_synonyms = new_synonyms + self.subsitutor(i)

        clear_text = list(set(new_synonyms + synonyms))
        encrypted = self.gen_enc_list(clear_text)

        return {'synonym': {'clear_text': clear_text, 'encrypted': encrypted}}

    def antonym(self, bio_val):
        """
        Generate antonyms of input data
        """
        antonyms = Antonym().generate_antonym(bio_val)
        antonyms = list(set(antonyms['results']))
        new_antonyms = []
        clear_text = []
        encrypted = {}

        for i in antonyms:
            new_antonyms = new_antonyms + self.subsitutor(i)

        clear_text = list(set(new_antonyms + antonyms))
        encrypted = self.gen_enc_list(clear_text)

        return {'antonyms': {'clear_text': clear_text, 'encrypted': encrypted}}

    def syzygy(self, bio_val):
        """
        Generate syzygy of input data
        """
        syzygys = Syzygy().generate_syzygy(bio_val)
        syzygys = list(set(syzygys['results']))
        new_syzygys = []
        clear_text = []
        encrypted = {}

        for i in syzygys:
            new_syzygys = new_syzygys + self.subsitutor(i)

        clear_text = list(set(new_syzygys + syzygys))
        encrypted = self.gen_enc_list(clear_text)

        return {'syzygys': {'clear_text': clear_text, 'encrypted': encrypted}}

    def anomaly(self, bio_val):
        """
        Generate anomaly of input data
        """
        anomalies = Anomaly().generate_anomaly(bio_val, self.words, 1)
        anomalies = list(set(anomalies['results']))
        new_anomalies = []
        clear_text = []
        encrypted = {}

        for i in anomalies:
            new_anomalies = new_anomalies + self.subsitutor(i)

        clear_text = list(set(new_anomalies + anomalies))
        encrypted = self.gen_enc_list(clear_text)

        return {
            'anomalies': {
                'clear_text': clear_text,
                'encrypted': encrypted}}

    def clinamen(self, bio_val):
        """
        Generate clinamen of input data
        """
        clinamen = Clinamen().generate_clinamen(bio_val, self.words, 1)
        clinamen = list(set(clinamen['results']))
        new_clinamen = []
        clear_text = []
        encrypted = {}

        for i in clinamen:
            new_clinamen = new_clinamen + self.subsitutor(i)

        clear_text = list(set(new_clinamen + clinamen))
        encrypted = self.gen_enc_list(clear_text)

        return {'clinamen': {'clear_text': clear_text, 'encrypted': encrypted}}

    def subsitutor(self, pwd):
        """
        Generate common character
        substitutions
        """
        new_pwds = []

        for s in self.substitutors_dict:
            generator_class = self.substitutors_dict[s]()
            new_pwds.append(generator_class.substitute(pwd))

        #mung_it = MungSubstitutor()
        # new_pwds.append(mung_it.total_mung_simple(pwd))
        # new_pwds.append(mung_it.random_mung_simple(pwd))

        return new_pwds



import yaml


class ProcessInputYaml():
    """
    Class to take an input YAML file
    of key words. Opens it and returns it
    for processing
    """

    def yaml_processor(self, yamldoc):
        """
        Open the YAML doc and return to
        caller
        """
        socialdata = {}
        opendoc = open(yamldoc, "r")
        socialdata = yaml.safe_load_all(opendoc)
        return socialdata


class ProcessInputWords():
    """
    Class to take an input word list and process
    it for use in pataphysial algos
    """

    def words_processor(self, worddoc):
        """
        Process input word list
        """
        worddata = []
        with open(worddoc) as file:
            for line in file:
                worddata.append(line.strip())
        return worddata



import hashlib


class SHA512Encryption():

    def hash(self, pwd):
        """
        Return a sha512 hash value
        """
        hash_val = hashlib.sha512()
        hash_val.update(pwd.encode('utf-8'))
        return hash_val.hexdigest()



from ..gen_password import PasswordGenerator
from ..date_name_mixin import DateNameMixin

class FamilyGenerator(DateNameMixin):
    """
    Class for generating 
    family based data.
    """
    family_data = {}
    cat = ""
    encryption_dict = {}
    substitutors_dict = {}
    words = []

    def process_data(
            self,
            k,
            words,
            family_bio_data,
            encryption_dict,
            substitutors_dict):
        """
        Process the family bio data
        """
        self.family_data = family_bio_data
        self.cat = k
        self.words = words
        self.encryption_dict = encryption_dict
        self.substitutors_dict = substitutors_dict

        family_passwords = self.process_individual()
        return family_passwords

    def process_individual(self):
        """
        Process an individuals data
        to generate a list of potential
        passwords.
        """
        individual = {}
        pata_passwords = []

        for ind in self.family_data:
            for bio, values in ind.iteritems():
                pata_passwords.append(self.name_dob(values))
                pata_passwords.append(
                    PasswordGenerator(
                        self.cat,
                        self.words,
                        values,
                        self.encryption_dict,
                        self.substitutors_dict).process_individual())
                individual[bio] = pata_passwords

            pata_passwords = []

        return individual




from setuptools import setup, find_packages
from setuptools.command.install import install
import io

class InstallNLTKDependencies(install):
    """
    Install any dependencies needed by
    the library from the NLTK installer
    for example corpora.
    """

    def run(self):
        """
        Add NLTK installation commands
        here
        """
        from nltk import wordnet
        print "Installing WordNet"
        install.run(self)


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


long_description = read('README.md')

setup(
    name='pata-password-cracker',
    version='1.1.0',
    description='Pataphysical passwork cracker using personal data',
    long_description=long_description,
    maintainer='patamechanix',
    license='MIT',
    url='https://github.com/patamechanix/pata-password-cracker',
    package_dir={'': 'src'},
    include_package_data=True,
    packages=find_packages('src'),
    entry_points={
        'console_script': [
            'pata_password_cracker = pata_password_cracker.__main__:main'
        ],
        'pata_password_cracker.plugins': [
            'core_bio = pata_password_cracker.generators.core_bio:CoreBioGenerator',
            'family = pata_password_cracker.generators.family:FamilyGenerator',
            'free_data = pata_password_cracker.generators.free_data:FreeDataGenerator'
        ],
        'pata_password_cracker.encryption': [
            'md5 = pata_password_cracker.encryption.md5:MD5Encryption',
            'sha1 = pata_password_cracker.encryption.sha1:SHA1Encryption',
            'sha224 = pata_password_cracker.encryption.sha224:SHA224Encryption',
            'sha256 = pata_password_cracker.encryption.sha256:SHA256Encryption',
            'sha384 = pata_password_cracker.encryption.sha384:SHA384Encryption',
            'sha512 = pata_password_cracker.encryption.sha512:SHA512Encryption',
            'bcrypt = pata_password_cracker.encryption.bcrypt:BcryptEncryption'
        ],
        'pata_password_cracker.substitutors': [
            'simple = pata_password_cracker.substitutors.simple:MungSubstitutor',
            'simplerandom = pata_password_cracker.substitutors.simplerandom:MungSubstitutorRandom',
            'common = pata_password_cracker.substitutors.common:MungSubstitutorCommon'
        ]
    },
    install_requires=[
        'patalib==1.0.1',
        'bcrypt'
    ],
    cmdclass={
        'install': InstallNLTKDependencies
    }
)



import pkg_resources


class Categories:
    """
    Load categories from YAML
    representation.
    """
    bio_data = {}
    password_mappings = {}
    individuals = {}
    category_plugin = "pata_password_cracker.plugins"
    encryption_plugin = "pata_password_cracker.encryption"
    substitutors_plugin = "pata_password_cracker.substitutors"
    loaded_cat_plugin_dict = {}
    loaded_encryption_plugin_dict = {}
    loaded_substitutors_plugin_dict = {}
    words = []

    def __init__(self, bio_data, words):
        """
        Store list of biographical data
        """
        self.bio_data = bio_data
        self.words = words
        self.loaded_cat_plugin_dict = self.load_plugins(self.category_plugin)
        self.loaded_encryption_plugin_dict = self.load_plugins(
            self.encryption_plugin)
        self.loaded_substitutors_plugin_dict = self.load_plugins(
            self.substitutors_plugin)

    def load_plugins(self, plugin):
        """
        Load the plugin and store object in array
        """
        plugin_dict = {}
        for p in pkg_resources.iter_entry_points(plugin):
            plugin_dict[p.name] = p.load()
        return plugin_dict

    def process_categories(self):
        """
        Process an individuals categories data
        to generate a list of potential
        passwords
        """
        individual = {}
        count = 0
        for target in self.bio_data:
            indv_key = str(count) + ":" + target
            indv_key = indv_key.replace(" ", "")
            individual[indv_key] = target
            target_vals = []
            count = count + 1

            for category in self.bio_data[target]:
                for k, v in category.iteritems():
                    for p in self.loaded_cat_plugin_dict:
                        if k == p:
                            target_vals.append(
                                {k: self.loaded_cat_plugin_dict[p]().process_data(
                                    k,
                                    self.words,
                                    v,
                                    self.loaded_encryption_plugin_dict,
                                    self.loaded_substitutors_plugin_dict)})
            individual[indv_key] = target_vals

        return individual



from ..gen_password import PasswordGenerator
from ..date_name_mixin import DateNameMixin


class CoreBioGenerator(DateNameMixin):
    """
    Generator for core biographic
    data
    """
    cat = ""
    encryption_dict = {}
    substitutors_dict = {}
    words = []

    def process_data(
            self,
            k,
            words,
            core_bio_data,
            encryption_dict,
            substitutors_dict):
        """
        Process the core bio data
        """
        self.cat = k
        self.words = words
        self.encryption_dict = encryption_dict
        self.substitutors_dict = substitutors_dict

        bio_passwords = []
        bio_passwords.append(self.name_dob(core_bio_data))
        bio_passwords.append(PasswordGenerator(
            k,
            words,
            core_bio_data,
            encryption_dict,
            substitutors_dict).process_individual())
        return bio_passwords
